#pragma once

#include "gl_types.h"

#define DLLFUNCTION(dll, output, name, inputs, params, required) \
  extern output (__stdcall *p##name) inputs

#include "gl_functions.h"

#undef DLLFUNCTION

// extensions
extern void (__stdcall *pglLockArraysEXT)(GLint first, GLsizei count);
extern void (__stdcall *pglUnlockArraysEXT)(void);

extern GLboolean (__stdcall *pwglSwapIntervalEXT)(GLint interval);
extern GLint     (__stdcall *pwglGetSwapIntervalEXT)(void);

extern void (__stdcall *pglActiveTextureARB)(GLenum texunit);
extern void (__stdcall *pglClientActiveTextureARB)(GLenum texunit);

// t-buffer support
extern char *(__stdcall *pwglGetExtensionsStringARB)(HDC hdc);
extern BOOL  (__stdcall *pwglChoosePixelFormatARB)(HDC hdc, const int *piAttribIList, const FLOAT *pfAttribFList, UINT nMaxFormats, int *piFormats, UINT *nNumFormats);
extern BOOL  (__stdcall *pwglGetPixelFormatAttribivARB)(HDC hdc, int iPixelFormat, int iLayerPlane, UINT nAttributes, int *piAttributes, int *piValues);
extern void  (__stdcall *pglTBufferMask3DFX)(GLuint mask);

// GL_NV_vertex_array_range & GL_NV_fence
extern void *(__stdcall *pwglAllocateMemoryNV)(GLint size, GLfloat readfreq, GLfloat writefreq, GLfloat priority);
extern void  (__stdcall *pwglFreeMemoryNV)(void *pointer);
extern void  (__stdcall *pglVertexArrayRangeNV)(GLsizei length, void *pointer);
extern void  (__stdcall *pglFlushVertexArrayRangeNV)(void);

extern GLboolean (__stdcall *pglTestFenceNV)(GLuint fence);
extern GLboolean (__stdcall *pglIsFenceNV)(GLuint fence);
extern void  (__stdcall *pglGenFencesNV)(GLsizei n, GLuint *fences);
extern void  (__stdcall *pglDeleteFencesNV)(GLsizei n, const GLuint *fences);
extern void  (__stdcall *pglSetFenceNV)(GLuint fence, GLenum condition);
extern void  (__stdcall *pglFinishFenceNV)(GLuint fence);
extern void  (__stdcall *pglGetFenceivNV)(GLuint fence, GLenum pname, GLint *params);



// additional tools -----------------------------------------------------

#include <Engine/Graphics/Color.h>
#include <Engine/Graphics/gl_vertex.h>
#include <Engine/Templates/StaticStackArray.cpp>


// set color from croteam format
inline void glCOLOR( COLOR col)
{
  __asm {
    mov     eax,dword ptr [col]
    bswap   eax
    mov     dword ptr [col],eax
  }
  pglColor4ubv((GLubyte*)&col);
}

// check windows errors always
extern void WIN_CheckError(BOOL bRes, const char *strDescription);
#define WIN_CHECKERROR(result, string)   WIN_CheckError(result, string);

// check gl errors only in debug builds
#ifndef NDEBUG
  extern void GL_CheckError(void);
  #define GL_CHECKERROR  GL_CheckError();
#else
  #define GL_CHECKERROR  (void)(0);
#endif

// maximum number of supported texture units for multitexturing
#define GFX_MAXTEXUNITS 8

// engine wrapers to eliminate call overhead in most used openGL functions
extern GLuint glaiBindNo[GFX_MAXTEXUNITS];
extern BOOL   glabTexture[GFX_MAXTEXUNITS];
extern BOOL   glbBlend;
extern BOOL   glbAlphaTest;
extern BOOL   glbDepthTest;
extern BOOL   glbDepthMask;
extern BOOL   glbUsingVARs;   // vertex_array_range
extern GLenum gleDepthFunc;
extern GLenum gleBlendSrc;
extern GLenum gleBlendDst;
extern INDEX  gliActiveTexture;        // zero-based!
extern INDEX  gliClientActiveTexture;  // zero-based!

extern INDEX ogl_bFastRendering;
extern INDEX ogl_bFastUploading;
extern INDEX ogl_bFastStateChange;


// common element arrays
extern CStaticStackArray<GLVertex4>  _avtxCommon;
extern CStaticStackArray<GLTexCoord> _atexCommon;
extern CStaticStackArray<GLColor>    _acolCommon;
extern CStaticStackArray<INDEX>      _aiCommonElements;
extern CStaticStackArray<INDEX>      _aiCommonQuads;

// common textures
extern GLuint _uiFillTextureNo;    // binding for flat fill emulator texture
extern GLuint _uiFogTextureNo;     // binding for fog texture
extern GLuint _uiHazeTextureNo;    // binding for haze texture
extern GLuint _uiPatternTextureNo; // binding for pattern texture



inline void eglActiveTexture( INDEX texunit)
{
  ASSERT( texunit>=0 && texunit<GFX_MAXTEXUNITS);
  ASSERT( pglActiveTextureARB!=NULL);
#ifndef NDEBUG
  GLuint currunit;
  pglGetIntegerv( GL_ACTIVE_TEXTURE_ARB, (GLint*)&currunit);
  GL_CHECKERROR;
  ASSERT( currunit==gliActiveTexture || currunit==(GL_TEXTURE0_ARB+gliActiveTexture)); // not sure about what OGL should return here
#endif
  if( gliActiveTexture==texunit && ogl_bFastStateChange) return;
  gliActiveTexture = texunit;
  pglActiveTextureARB( GLenum(GL_TEXTURE0_ARB+texunit));
}


inline void eglClientActiveTexture( INDEX texunit)
{
  ASSERT( texunit>=0 && texunit<GFX_MAXTEXUNITS);
  ASSERT( pglClientActiveTextureARB!=NULL);
#ifndef NDEBUG
  GLuint currunit;
  pglGetIntegerv( GL_CLIENT_ACTIVE_TEXTURE_ARB, (GLint*)&currunit);
  GL_CHECKERROR;
  ASSERT( currunit==gliClientActiveTexture || currunit==(GL_TEXTURE0_ARB+gliClientActiveTexture));
#endif
  if( gliClientActiveTexture==texunit && ogl_bFastStateChange) return;
  gliClientActiveTexture = texunit;
  pglClientActiveTextureARB(GLenum(GL_TEXTURE0_ARB+texunit));
}


inline void eglBindTexture( GLuint bindno)
{
#ifndef NDEBUG
  GLuint currbind;
  pglGetIntegerv( GL_TEXTURE_BINDING_2D, (GLint*)&currbind);
  GL_CHECKERROR;
  ASSERT( currbind==glaiBindNo[gliActiveTexture] || bindno==0);
#endif
  if( glaiBindNo[gliActiveTexture]==bindno && ogl_bFastStateChange && bindno!=0) return;
  glaiBindNo[gliActiveTexture]=bindno;
  pglBindTexture( GL_TEXTURE_2D, bindno);
}


// might reset current bindings when deleting textures
void eglDeleteTextures( GLsizei n, const GLuint *textures);


inline void eglEnableTexture(void)
{
  ASSERT( !pglIsEnabled( GL_TEXTURE_2D) == !glabTexture[gliActiveTexture]);
  if( glabTexture[gliActiveTexture] && ogl_bFastStateChange) return;
  glabTexture[gliActiveTexture] = GL_TRUE;
  pglEnable( GL_TEXTURE_2D);
  // make sure that binding isn't lost (drivers are like women - not sure!:)
  eglBindTexture(0);
  GL_CHECKERROR;
}

inline void eglDisableTexture(void)
{
  ASSERT( !pglIsEnabled( GL_TEXTURE_2D) == !glabTexture[gliActiveTexture]);
  if( !glabTexture[gliActiveTexture] && ogl_bFastStateChange) return;
  glabTexture[gliActiveTexture] = GL_FALSE;
  pglDisable( GL_TEXTURE_2D);
}


inline void eglEnableBlend(void)
{
  ASSERT( pglIsEnabled( GL_BLEND) == glbBlend);
  if( glbBlend && ogl_bFastStateChange) return;
  glbBlend = GL_TRUE;
  pglEnable( GL_BLEND);
}

inline void eglDisableBlend(void)
{
  ASSERT( pglIsEnabled( GL_BLEND) == glbBlend);
  if( !glbBlend && ogl_bFastStateChange) return;
  glbBlend = GL_FALSE;
  pglDisable( GL_BLEND);
}


inline void eglEnableDepthTest(void)
{
  ASSERT( pglIsEnabled( GL_DEPTH_TEST) == glbDepthTest);
  if( glbDepthTest && ogl_bFastStateChange) return;
  glbDepthTest = GL_TRUE;
  pglEnable( GL_DEPTH_TEST);
}

inline void eglDisableDepthTest(void)
{
  ASSERT( pglIsEnabled( GL_DEPTH_TEST) == glbDepthTest);
  if( !glbDepthTest && ogl_bFastStateChange) return;
  glbDepthTest = GL_FALSE;
  pglDisable( GL_DEPTH_TEST);
}


inline void eglEnableAlphaTest(void)
{
  ASSERT( pglIsEnabled( GL_ALPHA_TEST) == glbAlphaTest);
  if( glbAlphaTest && ogl_bFastStateChange) return;
  glbAlphaTest = GL_TRUE;
  pglEnable( GL_ALPHA_TEST);
}

inline void eglDisableAlphaTest(void)
{
  ASSERT( pglIsEnabled( GL_ALPHA_TEST) == glbAlphaTest);
  if( !glbAlphaTest && ogl_bFastStateChange) return;
  glbAlphaTest = GL_FALSE;
  pglDisable( GL_ALPHA_TEST);
}


inline void eglBlendFunc( GLenum src, GLenum dst)
{
#ifndef NDEBUG
  GLenum gleSrc, gleDst;
  pglGetIntegerv( GL_BLEND_SRC, (GLint*)&gleSrc);
  pglGetIntegerv( GL_BLEND_DST, (GLint*)&gleDst);
  GL_CHECKERROR;
  ASSERT( gleSrc==gleBlendSrc && gleDst==gleBlendDst);
#endif
  if( gleBlendSrc==src && gleBlendDst==dst && ogl_bFastStateChange) return;
  gleBlendSrc=src;
  gleBlendDst=dst;
  pglBlendFunc( src, dst);
}


inline void eglDepthMask( GLboolean flag)
{
#ifndef NDEBUG
  GLboolean glbMask;
  pglGetIntegerv( GL_DEPTH_WRITEMASK, (GLint*)&glbMask);
  GL_CHECKERROR;
  ASSERT( glbMask==glbDepthMask);
#endif
  if( glbDepthMask==flag && ogl_bFastStateChange) return;
  glbDepthMask=flag;
  pglDepthMask( flag);
}

inline void eglDepthFunc( GLenum func)
{
#ifndef NDEBUG
  GLenum gleFunc;
  pglGetIntegerv( GL_DEPTH_FUNC, (GLint*)&gleFunc);
  ASSERT( gleFunc==gleDepthFunc);
#endif
  if( gleDepthFunc==func && ogl_bFastStateChange) return;
  gleDepthFunc=func;
  pglDepthFunc( func);
}


// compiled vertex array control
extern void eglLockArrays( const GLsizei count, const BOOL bLock=TRUE);
extern void eglUnlockArrays(void);
extern void eglDrawElements( const GLsizei count, const GLvoid *indices);

// vertex buffers control
extern void  eglStartVertexBlock(void); // start preparing new vertex block (fence)
extern void  eglEndVertexBlock(  void); // finished preparing of new vertex block (fence)
extern void *eglGetVertexMemory( INDEX ctBytes, BOOL b1stPass); // get memory for vertices and/or only one tex/color pass
extern void  eglAdjustVertexBuffers( SLONG slWantedBytes);  // adjust vertex memory buffer size and type
extern void  eglSetVertexArrayRange( BOOL bState);  // enable/disable usage of enhanced nVidia's HW T&L extension


// helper functions for drawing simple primitives thru drawelements


inline void eglResetArrays(void)
{
  _avtxCommon.PopAll();
  _atexCommon.PopAll();
  _acolCommon.PopAll();
  _aiCommonElements.PopAll();
}
  
// render elements to screen buffer
extern void eglFlushElements(void);
extern void eglFlushQuads(void);
