#pragma once

#include <Engine/Base/Profiling.h>

/* Class for holding profiling information for models. */
class CModelProfile : public CProfileForm {
public:
  // indices for profiling counters and timers
  enum ProfileTimerIndex {
    PTI_INITMODELRENDERING,
    PTI_INITPROJECTION,
    PTI_INITSHADOWPROJECTION,
    PTI_INITATTACHMENTS,
    PTI_CREATEATTACHMENT,
    PTI_RENDERMODEL,
    PTI_RENDERSHADOW,
    PTI_RENDERSIMPLESHADOW,

    PTI_OPENGL_PREPAREFORRENDERING,
    PTI_OPENGL_SETMODELVIEW,
    PTI_OPENGL_INIT_UNPACK,
    PTI_OPENGL_INIT_VERTICES,
    PTI_OPENGL_INIT_FOG_MIP,       
    PTI_OPENGL_INIT_HAZE_MIP,       
    PTI_OPENGL_RENDERMODEL,
    PTI_OPENGL_INIT_DIFF_SURF,
    PTI_OPENGL_INIT_BUMP_SURF,
    PTI_OPENGL_INIT_REFL_MIP ,
    PTI_OPENGL_INIT_REFL_SURF,
    PTI_OPENGL_INIT_SPEC_MIP, 
    PTI_OPENGL_INIT_SPEC_SURF,
    PTI_OPENGL_INIT_FOG_SURF,      
    PTI_OPENGL_INIT_HAZE_SURF,      
    PTI_OPENGL_ATTENUATE_SURF,

    PTI_OPENGL_RENDER_DIFFUSE,
    PTI_OPENGL_RENDER_BUMP,
    PTI_OPENGL_RENDER_REFLECTIONS,
    PTI_OPENGL_RENDER_SPECULAR,
    PTI_OPENGL_RENDER_FOG,          
    PTI_OPENGL_RENDER_HAZE,         

    PTI_OPENGL_SETTEXTURE,
    PTI_OPENGL_LOCKARRAYS,
    PTI_OPENGL_DRAWELEMENTS,
    PTI_OPENGL_ONESIDE,
    PTI_OPENGL_ONESIDE_GLSETUP,

    PTI_OPENGL_RENDERSHADOW,
    PTI_OPENGL_SHAD_INIT_MIP,
    PTI_OPENGL_SHAD_INIT_SURF,
    PTI_OPENGL_SHAD_GLSETUP,
    PTI_OPENGL_SHAD_RENDER,
    PTI_OPENGL_RENDERSIMPLESHADOW,
    PTI_OPENGL_SIMP_CALC,
    PTI_OPENGL_SIMP_PREP,
    PTI_OPENGL_SIMP_COPY,
    PTI_OPENGL_SIMP_BATCHED,

    PTI_OPENGL_RENDERPATCHES,

    PTI_MASK_INITMODELRENDERING,
    PTI_MASK_RENDERMODEL,

    PTI_COUNT
  };

  enum ProfileCounterIndex {

    PCI_VERTICES_FIRSTMIP,
    PCI_VERTICES_USEDMIP,
    PCI_SURFACEVERTICES_FIRSTMIP,
    PCI_SURFACEVERTICES_USEDMIP,
    PCI_TRIANGLES_FIRSTMIP,
    PCI_TRIANGLES_USEDMIP,

    PCI_SHADOWVERTICES_FIRSTMIP,
    PCI_SHADOWVERTICES_USEDMIP,
    PCI_SHADOWSURFACEVERTICES_FIRSTMIP,
    PCI_SHADOWSURFACEVERTICES_USEDMIP,
    PCI_SHADOWTRIANGLES_FIRSTMIP,
    PCI_SHADOWTRIANGLES_USEDMIP,

    PCI_OPENGL_TRIANGLES,

    PCI_MASK_TRIANGLES,
    PCI_MASK_POLYGONS,

    PCI_COUNT
  };
  // constructor
  CModelProfile(void);
  // override to provide external averaging from gfx profile
  virtual INDEX GetAveragingCounter(void);
};
