#pragma once

/*
 *  PCM Wave Input
 */
class PCMWaveInput {
private:
  // Wave data
  WAVEFORMATEX    pwi_SwfeWave;
  WAVEFORMATEX    pwi_SwfeDesired;
  ULONG           pwi_ulRiffLength;
  ULONG           pwi_ulDataLength;
  // Status
  BOOL    pwi_bInfoLoaded;
  BOOL    pwi_bDataLoaded;
  // Memory
  void    *pwi_pvMemory;

  /* Conversion */
  double  pwi_dRatio;
  // get and store data
  inline ULONG   GetData_t(CTStream *pCstrInput);
  inline void    StoreData(ULONG ulData);
  // copy data
  void    CopyData_t(CTStream *pCstrInput);
  // expand data
  void    ExpandData_t(CTStream *pCstrInput);
  void    ExpandMono(ULONG ulDataFrom, ULONG ulDataTo);
  void    ExpandStereo(ULONG ulLDataFrom, ULONG ulRDataFrom, ULONG ulLDataTo, ULONG ulRDataTo);
  // shrink data
  void    ShrinkData_t(CTStream *pCstrInput);

public:
  // Check wave format
  static void CheckWaveFormat_t(WAVEFORMATEX SwfeCheck, char *pcErrorString);

  /* Constructor */
  inline  PCMWaveInput(void) {pwi_bInfoLoaded = FALSE; pwi_bDataLoaded = FALSE;};
  /* Load Wave info */
  WAVEFORMATEX LoadInfo_t(CTStream *pCstrInput);
  /* Load and convert Wave data */
  void    LoadData_t(CTStream *pCstrInput, void *pvMemory, WAVEFORMATEX SwfeDesired);
  /* Load Wave data */
  void    LoadData_t(CTStream *pCstrInput, void *pvMemory);

  /* Length in bytes / blocks / seconds */
  ULONG   GetByteLength(void);
  ULONG   GetDataLength(void);
  ULONG   GetDataLength(WAVEFORMATEX SwfeDesired);
  double  GetSecondsLength(void);

  /* Buffer length in bytes */
  ULONG   DetermineBufferSize(void);
  ULONG   DetermineBufferSize(WAVEFORMATEX SwfeDesired);
};

